#  Author:   Adel Benlagra  <abenlagra@rocketscience.one>

from abc import ABC, abstractmethod
from datetime import datetime
from enum import Enum

import pandas as pd
from pydantic import BaseModel, validator


class MetricsType(Enum):

    """Metrics type enumeration"""

    performance = "performance"
    drift = "drift"
    custom = "custom"


class MetricResults(BaseModel):

    """Data structure for the results of a metric"""

    metric_name: str = None
    type: str = None
    model_id: str = None
    model_version: str = None
    data_id: str = None
    feature: str = None
    value: float = None
    status: bool = None
    threshold: float = None
    period_start: datetime = None
    period_end: datetime = None
    eval_timestamp: datetime = datetime.now()
    conf_int: list = None

    @validator("eval_timestamp", always=True)
    def timestamp_later_than_period_end(cls, v, values, **kwargs):
        if v < values["period_end"]:
            raise ValueError("Current timestamp earlier than period end")
        return v

    # TODO: validators for model id's, model's version, data_id, and metrics type


class AbstractMetrics(ABC):

    """Base abstract class for metricsa"""

    def __init__(self, name: str, data: pd.DataFrame):

        """Parameters
        ----------
        - name: name of the metrics
        - data: dataset from which the metric is calculated
        """

        self._name = name
        self._data = data.copy(deep=True)  # Not sure I want to attach the data as an attribute ...

        # TODO: validation on the dataset ?

        try:
            # TODO: better handling of date format
            data["pred_timestamp"] = pd.to_datetime(data["pred_timestamp"])
            self._model_id = str(data["model_id"].unique()[0])
            self._model_version = str(data["model_version"].unique()[0])
            self._period_start = data.pred_timestamp.min()
            self._period_end = data.pred_timestamp.max()
            self._result = None
        except Exception as e:
            print(str(e))

    @property
    @abstractmethod
    def evaluate(self) -> MetricResults:
        raise NotImplementedError

    def _check_metrics_name(self):
        pass

    def get_result(self):
        return self._result

    # TODO: method to compare the metrics value to single value or interval thresholds


def CustomMetric(func):
    """Decorator for custom metrics"""

    def inner(name: str, data: pd.DataFrame) -> AbstractMetrics:
        class CustomClass(AbstractMetrics):
            def __init__(self, name, data):
                super().__init__(name, data)

            def evaluate(self, **kwargs):

                value = func(**kwargs)
                threshold = kwargs.get("threshold", None)

                if isinstance(threshold, (int, float)):
                    status = value < threshold
                else:
                    status = None

                self._result = MetricResults(
                    metric_name=self._name,
                    type=MetricsType.custom.value,
                    model_id=self._model_id,
                    model_version=self._model_version,
                    value=value,
                    conf_int=None,
                    status=status,
                    threshold=threshold,
                    period_start=self._period_start,
                    period_end=self._period_end,
                )

                return self._result

        return CustomClass(name=name, data=data)

    return inner



#  Author:   Adel Benlagra  <abenlagra@rocketscience.one>

from enum import Enum
from functools import partial

import pandas as pd
from black import InvalidInput
from scipy.stats import (
    chisquare,
    cramervonmises_2samp,
    ks_2samp,
    levene,
    mannwhitneyu,
    ttest_ind,
    wasserstein_distance,
)

from .base import AbstractMetrics, MetricResults, MetricsType
from .utils import kl_divergence


class DriftMetricsFuncs(Enum):
    kl = partial(kl_divergence)
    wasserstein = partial(wasserstein_distance)


class DriftTestMetricsFuncs(Enum):
    ttest = partial(ttest_ind, equal_var=False)
    manwu = partial(mannwhitneyu)
    levene = partial(levene, center="mean")
    bftest = partial(levene, center="median")
    ks_2samp = partial(ks_2samp)
    CvM = partial(cramervonmises_2samp)
    chi2 = partial(chisquare)


class DriftMetric(AbstractMetrics):
    def __init__(self, name: str, data: pd.DataFrame, feature_name: str, **kwargs):

        """Supercharged init method for drift metrics"""

        super().__init__(name, data)

        self._check_metrics_name(name)

        try:
            self._feature_name = feature_name
            self._column = data[feature_name]

        except Exception as e:
            print(str(e))

    def _check_metrics_name(self, name: str):
        if name not in DriftMetricsFuncs._member_names_:
            raise InvalidInput(f"unknown metric key '{name}' given. " f"Should be one of {DriftMetricsFuncs._member_names_}.")

    def evaluate(
        self,
        reference: pd.Series,
        threshold: float = None,
        **kwargs,
    ) -> MetricResults:

        """Evaluation function for performance metrics

        Parameters
        ----------
        - bootstrap (bool): whether to bootstrap the metric for confidence interval calculation
        - n_bootstrap (int): number of bootstrapping samples
        - seed (int): seed for random number generator
        - alpha (float): significance level
        - **kwargs: parameters of the function used to calculate the metric
        """

        try:

            value = DriftMetricsFuncs[self._name].value(self._column, reference, **kwargs)

            if isinstance(threshold, (int, float)):
                status = value < threshold
            else:
                status = None

            self._result = MetricResults(
                metric_name=self._name,
                type=MetricsType.drift.value,
                model_id=self._model_id,
                model_version=self._model_version,
                feature=self._feature_name,
                value=value,
                conf_int=None,
                status=status,
                threshold=threshold,
                period_start=self._period_start,
                period_end=self._period_end,
            )

            return self._result

        except Exception as e:
            print(str(e))


class DriftTestMetric(AbstractMetrics):
    def __init__(self, name: str, data: pd.DataFrame, feature_name: str, **kwargs):

        """Supercharged init method for drift metrics"""

        super().__init__(name, data)

        self._check_metrics_name(name)

        try:
            self._feature_name = feature_name
            self._column = data[feature_name]

        except Exception as e:
            print(str(e))

    def _check_metrics_name(self, name: str):
        if name not in DriftTestMetricsFuncs._member_names_:
            raise InvalidInput(f"unknown metric key '{name}' given. " f"Should be one of {DriftTestMetricsFuncs._member_names_}.")

    def evaluate(
        self,
        reference: pd.Series,
        alpha: float = 0.05,
        **kwargs,
    ) -> MetricResults:

        """Evaluation function for performance metrics

        Parameters
        ----------
        - bootstrap (bool): whether to bootstrap the metric for confidence interval calculation
        - n_bootstrap (int): number of bootstrapping samples
        - seed (int): seed for random number generator
        - alpha (float): significance level
        - **kwargs: parameters of the function used to calculate the metric
        """

        try:

            if self._name != "CvM":
                _, pvalue = DriftTestMetricsFuncs[self._name].value(self._column, reference, **kwargs)
            else:
                res = DriftTestMetricsFuncs[self._name].value(self._column, reference, **kwargs)
                # statistic = res.statistic
                pvalue = res.pvalue

            if isinstance(alpha, (int, float)):
                status = pvalue < alpha
            else:
                status = None

            self._result = MetricResults(
                metric_name=self._name,
                type=MetricsType.drift.value,
                model_id=self._model_id,
                model_version=self._model_version,
                feature=self._feature_name,
                value=pvalue,
                conf_int=None,
                status=status,
                threshold=alpha,
                period_start=self._period_start,
                period_end=self._period_end,
            )

            return self._result

        except Exception as e:
            print(str(e))



#  Author:   Adel Benlagra  <abenlagra@rocketscience.one>

import numpy as np
import pandas as pd
from scipy.stats import entropy


def kl_divergence(new: pd.Series, reference: pd.Series):

    # Getting the same bins for both columns when they are numeric
    if (reference.dtype == float) & (new.dtype == float):
        bins = np.histogram_bin_edges(np.concatenate([reference, new]), bins="sturges")
        reference = pd.cut(reference, bins)
        new = pd.cut(new, bins)

    vector_all = pd.concat([reference, new], keys=["ref", "new"]).reset_index(0)

    percents = vector_all.groupby("level_0").value_counts(normalize=True).sort_index().unstack().T

    kl_div = entropy(percents["new"], percents["ref"])

    return kl_div



#  Author:   Adel Benlagra  <abenlagra@rocketscience.one>

from enum import Enum
from functools import partial

import numpy as np
import pandas as pd
from black import InvalidInput
from sklearn.metrics import (
    accuracy_score,
    average_precision_score,
    brier_score_loss,
    f1_score,
    log_loss,
    mean_absolute_error,
    mean_squared_error,
    precision_score,
    r2_score,
    recall_score,
    roc_auc_score,
)

from .base import AbstractMetrics, MetricResults, MetricsType


class PerformanceMetricsFuncs(Enum):

    """Set of performance metrics functions"""

    # Classification Metrics
    accuracy = partial(accuracy_score)
    precision = partial(precision_score)
    recall = partial(recall_score)
    f1 = partial(f1_score)
    log_loss = partial(log_loss)
    # Area under ROC Curve
    auc = partial(roc_auc_score)
    # Area under PR Curve
    aucpr = partial(average_precision_score)
    brier = partial(brier_score_loss)

    # Regression metrics
    mse = partial(mean_squared_error)
    mae = partial(mean_absolute_error)
    mape = partial(mean_absolute_error)
    r2 = partial(r2_score)


class PerformanceMetric(AbstractMetrics):
    def __init__(self, name: str, data: pd.DataFrame, **kwargs):

        """Supercharged init method for performance metrics"""

        super().__init__(name, data)

        self._check_metrics_name(name)

        try:
            y_name = kwargs.get("y_name", "y_true")
            pred_name = kwargs.get("pred_name", "y_pred")
            self._y_true = data[y_name]
            self._y_pred = data[pred_name]

        except Exception as e:
            print(str(e))

    def _check_metrics_name(self, name: str):
        if name not in PerformanceMetricsFuncs._member_names_:
            raise InvalidInput(
                f"unknown metric key '{name}' given. " f"Should be one of {PerformanceMetricsFuncs._member_names_}."
            )

    def evaluate(
        self,
        bootstrap: bool = False,
        n_bootstrap: int = 100,
        alpha: float = 0.05,
        seed: int = 123,
        threshold: float = None,
        **kwargs,
    ) -> MetricResults:

        """Evaluation function for performance metrics

        Parameters
        ----------
        - bootstrap (bool): whether to bootstrap the metric for confidence interval calculation
        - n_bootstrap (int): number of bootstrapping samples
        - seed (int): seed for random number generator
        - alpha (float): significance level
        - **kwargs: parameters of the function used to calculate the metric
        """

        try:

            value = PerformanceMetricsFuncs[self._name].value(self._y_true, self._y_pred, **kwargs)

            if bootstrap:
                conf_int = self._bootstrap(n_bootstrap=n_bootstrap, alpha=alpha, seed=seed, **kwargs)
            else:
                conf_int = None

            if isinstance(threshold, (int, float)):
                status = value < threshold
            else:
                status = None

            self._result = MetricResults(
                metric_name=self._name,
                type=MetricsType.performance.value,
                model_id=self._model_id,
                model_version=self._model_version,
                value=value,
                feature="prediction",
                conf_int=conf_int,
                status=status,
                threshold=threshold,
                period_start=self._period_start,
                period_end=self._period_end,
            )

            return self._result

        except Exception as e:
            print(str(e))

    def _bootstrap(self, n_bootstrap: int = 100, seed: int = 123, alpha: float = 0.05, **kwargs):

        """Function to bootstrap the metrics for confidence interval evaluation

        Parameters
        ----------
        - n_bootstrap (int): number of bootstrapping samples
        - seed (int): seed for random number generator
        - alpha (float): significance level
        """

        rng = np.random.default_rng(seed)
        n = self._data.shape[0]

        values = []

        for i in range(n_bootstrap):
            indices = rng.integers(low=0, high=n, size=n)
            values.append(
                PerformanceMetricsFuncs[self._name].value(self._y_true.loc[indices], self._y_pred.loc[indices], **kwargs)
            )
        return [np.quantile(values, alpha / 2), np.quantile(values, 1 - alpha / 2)]



import pandas as pd


def validate_dataframe(data: pd.DataFrame, y_name: str = "y_true", pred_name: str = "y_pred"):

    if y_name not in data.columns:
        raise KeyError("The name of the realized target is not in the dataset")
    elif pred_name not in data.columns:
        raise KeyError("The name of the realized target is not in the dataset")
    elif "date" not in data.columns:
        raise KeyError('There is not datetime column in the dataset or it should be named "date"')
    elif "model_id" not in data.columns:
        raise KeyError('There is not model id column in the dataset or it should be named "model_id"')
    else:
        print("The dataframe is validated !")
        return True



#  Author:   Adel Benlagra  <abenlagra@rocketscience.one>

from abc import ABC, abstractmethod

import pandas as pd
from black import InvalidInput

from ..metrics.drift import (
    DriftMetric,
    DriftMetricsFuncs,
    DriftTestMetric,
    DriftTestMetricsFuncs,
)
from ..metrics.performance import PerformanceMetric


class AbstractAnalyzer(ABC):

    """Base abstract class for analyzers"""

    def __init__(self, name: str, data: pd.DataFrame, description: str = None):

        """Parameters
        ----------
        - name: name of the analyzer
        - data: dataset from which the abnalyzer is defined
        - description (Optional): description for the analyzer
        """

        self._name = name
        self._data = data.copy(deep=True)  # Not sure I want to attach the data as an attribute ...
        self._description = description

        # TODO: validation on the dataset ?

        try:
            # TODO: better handling of date format
            data["pred_timestamp"] = pd.to_datetime(data["pred_timestamp"])
            self._model_id = str(data["model_id"].unique()[0])
            self._model_version = str(data["model_version"].unique()[0])
            self._period_start = data.pred_timestamp.min()
            self._period_end = data.pred_timestamp.max()
            self._metrics_list = []
            self._results = None
        except Exception as e:
            print(str(e))

    @property
    @abstractmethod
    def run(self, data_ref: pd.DataFrame):
        raise NotImplementedError

    def schedule(self):
        pass

    def add_performance_metrics(self, metrics_list: list):
        pass

    def add_drift_metrics(self, metrics_list: list):
        pass

    def add_drift_test_metrics(self, metrics_list: list):
        pass

    def get_result(self):
        return self._results

    def results_to_json(self):
        if self._results is None:
            return {}
        else:
            return [result.json() for result in self._results]

    def results_to_pandas(self):
        if self._results is None:
            return None
        else:
            return pd.DataFrame.from_records([self._results[i].dict() for i in range(len(self._results))])

    def log_results(self):
        pass


class Analyzer(AbstractAnalyzer):
    def __init__(self, name: str, data: pd.DataFrame, description: str = None, **kwargs):

        """Supercharged init method for performance metrics"""

        super().__init__(name, data, description)

    def add_performance_metrics(self, metrics_list: list, **kwargs):

        """Adding a list of performance metrics to the analyzer"""

        """Parameters
        ----------
        - metrics_list: list of performance metrics names
        """

        for metric_name in metrics_list:
            try:
                metric = PerformanceMetric(name=metric_name, data=self._data, **kwargs)
                if hasattr(metric, "_y_true"):
                    self._metrics_list.append(metric)
                    print("Performance metric '{}' added to the analyzer list".format(metric_name))
                else:
                    raise ValueError(
                        f"The dataset contains no ground truth for performance assessment. Metric '{metric_name}'"
                        f"was NOT added to the analyzer list"
                    )
            except Exception as e:
                print(str(e))

    def add_drift_metrics(self, metrics_list: list, features_list: list = None):

        """Adding a list of drift metrics to the analyzer"""

        """Parameters
        ----------
        - metrics_list: list of drift metric names
        """

        if features_list is None:
            features_list = self._data.select_dtypes("number").columns

        for metric_name in metrics_list:
            for feature in features_list:
                try:
                    if metric_name in DriftMetricsFuncs._member_names_:
                        metric = DriftMetric(name=metric_name, data=self._data, feature_name=feature)
                    elif metric_name in DriftTestMetricsFuncs._member_names_:
                        metric = DriftTestMetric(name=metric_name, data=self._data, feature_name=feature)
                    else:
                        metric = None
                        raise InvalidInput(f"unknown drift metric key '{metric_name}' given. ")
                    if metric is not None:
                        self._metrics_list.append(metric)
                        print("Drift metric '{}' for feature '{}' added to the analyzer list".format(metric_name, feature))
                except Exception as e:
                    print(str(e))

    def run(self, data_ref: pd.DataFrame, options: dict = {}):

        """Running the analyzer from the list of metrics"""

        if len(self._metrics_list) == 0:
            raise ValueError("The list of metrics for the analyzer is empty.")
        else:
            try:
                self._results = []
                for metric in self._metrics_list:
                    kwargs = options.get(metric._name, {})
                    if isinstance(metric, (DriftMetric, DriftTestMetric)):
                        metric.evaluate(reference=data_ref[metric._feature_name], **kwargs)
                    elif isinstance(metric, PerformanceMetric):
                        metric.evaluate(**kwargs)

                    self._results.append(metric._result)
            except Exception as e:
                print(str(e))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pulsar_metrics', 'pulsar_metrics.analyzers', 'pulsar_metrics.metrics']

package_data = \
{'': ['*']}

install_requires = \
['DateTime>=4.5,<5.0',
 'black>=22.6.0,<23.0.0',
 'pandas>=1.4.3,<2.0.0',
 'pydantic>=1.9.1,<2.0.0',
 'scikit-learn>=1.1.1,<2.0.0',
 'scipy>=1.7,<2.0']

setup_kwargs = {
    'name': 'pulsar-metrics',
    'version': '0.1.2',
    'description': 'Pulsar-metrics is an open-source Python library for evaluating and monitoring data and concept drift with an extensive set of metrics. It also offers the possibility to use custom metrics defined by the user.',
    'long_description': '# pulsar-metrics\n\nPulsar-metrics is an open-source Python library for evaluating and monitoring data and concept drift with an extensive set of metrics. It also offers the possibility to use custom metrics defined by the user.\n\n## Getting started\n\n### Pulsar-metrics components\n\nThere are two core components in pulsar-metrics: metrics and analyzers\n\n#### Metrics\nAn API to calculate single metrics for data and concept drift. Metrics results are unified in a single data structure `MetricsResults` storing not only the metrics value, but also a couple of metadata related to the model and the data used for its calculation\n\n```\nMetricResults(metric_name=None, type=\'performance\', model_id=\'model_1\', model_version=\'1\', data_id=None, feature=None, value=None, status=None, threshold=None, period_start=None, period_end=datetime.datetime(2022, 7, 1, 0, 0), eval_timestamp=datetime.datetime(2022, 9, 26, 10, 28, 27, 846122), conf_int=None)\n```\n\nThere are three types of metrics:\n\n##### - Data drift metrics for the calculation of ditributional changes of the features used in the model. The metrics included so far are:\n###### [Kullback-Leibler (KL) divergence](https://en.wikipedia.org/wiki/Kullback–Leibler_divergence): This statistics measures how different is a probability distribution $P$ with respect to a reference probability distributiuon $Q$ (typically the probability distribution of the treaining features). More precisely, the KL divergence $D_{KL}(P||Q)$ is given by the fllowing formula $$D_{KL}(P||Q) = \\sum_x P(x) \\log \\left ( \\frac{P(x)}{Q(x)} \\right )$$ $D_{KL}(P||Q)$ is always non-negative et is zero when the distributions are identical. Hence, a drift would be detected if its value is larger than a given threshold decided by the use\n\n###### [Wasserstein distance](https://en.wikipedia.org/wiki/Wasserstein_metric) is a distance measure between two probability measures $Q$ and $P$. More precisely, the (first) Wassersetin distance $W_1(P, Q)$ is given by the formula $$W_1(P, Q) = \\int_{-\\infty}^{+\\infty}|F_Q(x) - F_P(x)|dx$$ where $F_Q$ is the cumulative distribution function of $Q$. The metric is strctly non negative and a drift would be detected if its value is larger than a given threshold decided by the user.\n\n###### [T-test](https://en.wikipedia.org/wiki/Student%27s_t-test) is a 2 samples paremetric statistical test to detect a difference in the means of the distributions of the two samples. More precisely, the test used is the [Welch test](https://en.wikipedia.org/wiki/Welch%27s_t-test) in which the 2 samples do not necessarily have the same variance or size. Since it is a statistical test, a location drift is detected when the p-value is smaller than a significance level chosen by the user (default is 0.05).\n\n###### [Mann-Whitney U test](https://en.wikipedia.org/wiki/Mann–Whitney_U_test) for location shift is a 2 sample non-parametric statistical test to detect a difference in the medians of the distrbutions of two samples. Since it is a statistical test, a location drift is detected when the p-value is smaller than a significance level chosen by the user (default is 0.05).\n\n###### [Levene\'s test](https://en.wikipedia.org/wiki/Levene%27s_test) is a 2 samples parametric statistical test to detect a difference in the variances of the distributions of the two samples. Since it is a statistical test, a dispersion drift is detected when the p-value is smaller than a significance level chosen by the user (default is 0.05).\n\n###### [Kolmogorov-Smirnov test](https://en.wikipedia.org/wiki/Kolmogorov–Smirnov_test) is a 2 samples nonparametric statistical test to check whether two samples come from the same distribution. The test statistics is given by $$D_{n, m} = \\sup_x |F_{1, n}(x) - F_{2, m}(x)|$$ where $F_{1, m}$ is the empirical cumulative distrbutin functin of sample 1 with size $n$. Since it is a statistical test, a dispersion drift is detected when the p-value is smaller than a significance level chosen by the user (default is 0.05).\n\n###### [Cramer von Mises test](https://en.wikipedia.org/wiki/Cramér–von_Mises_criterion) is a 2 samples nonparametric statistical test to check whether two samples come from the same distribution. The test statistics is given by $$T_{n, m} = \\frac{nm}{n+m} \\int_{-\\infty}^{+\\infty} |F_{1, n}(x) - F_{2, m}(x)|^2 dF_{n+m}$$ where $F_{1, m}$ is the empirical cumulative distrbutin functin of sample 1 with size $n$ and $F_{n+m}$ is the emprirical distribution function of the two samples together. Since it is a statistical test, a distributios drift is detected when the p-value is smaller than a significance level chosen by the user (default is 0.05).\n\n###### [Chi-square test](https://en.wikipedia.org/wiki/Chi-squared_test) to compare the distribution of a categorical feature in 2 samples by comparing the frequencies of unique modalities. Since it is a statistical test, a distribution drift is detected when the p-value is smaller than a significance level chosen by the user (default is 0.05).\n\nData drift metrics are implemented either in the `DriftMetric` (For the KL divergence and the Wasserstein distance) or `DriftTestMetric` classes. The choice of the metric is specified with the `name` parameter in the init method according to the following table\n\n|Metric|Name|\n|------|----|\n|Kullback-Leibler divergence|\'kl\'|\n|Wasserstein distance|\'wasserstein\'|\n|T-test|\'ttest\'|\n|Mann Whitney U test|\'manwu\'|\n|Leven\'s test|\'levene\'|\n|Kolmgorv-Smirnov|\'ks_2samp\'|\n|Cramer von Mises test|\'CvM\'|\n|Chi square test|\'chi2\'|\n\n\n##### - Performance metrics for te calculation of the performance of classification and regression models. In particular, the following metrics are implemented:\n\n* Accuracy\n* Precision\n* Recall\n* f1-score\n* Log loss\n* AUC\n* AUCPR\n* Brier Score\n* Mean squarred error (MSE)\n* Mean absolute error (MAE)\n* Mean absolute percentage error (MAPE)\n* R-square score\n\nPerformance metrics are implemented in the `PerformanceMetric` class.\n\n##### - Custom metrics. The user has the ability to define his own metric through the `@CustomMetric` decorator (see below for an example)\n\nAll three types of metrics inherit the `AbstractMetrics` class.\n#### Analyzers\n\nAn analyzer groups multiple metrics calculations in a single run. It allows to use which metrics to use and for which features.\n\n### Example usage\n\nTo use the library, you need a reference dataset, typically the training dataset, and an analysis dataset which we want to compare with former.\n\n\n#### Calculating a single metric\nFor a single metric, we first start by instantiating the appropriate metrics class by specifying the name of the metric ("ttest" in the example below)\n\n```python\nfrom pulsar_metrics.metrics.drift import DriftTestMetric\ndriftTest = DriftTestMetric(name = \'ttest\', data = data_new, feature_name = feature_name)\n```\nThen we run the `.evaluate()` method to calculate the metric\n\n```python\ndriftTest.evaluate(alpha = 0.05, reference = data_ref[feature_name])\n```\n\nThe result is returned through the `.get_result()` method of te metric object\n\n```python\ndriftTest.get_result()\n```\n\n#### Using the analyzer\nWhen multiple metrics are required for different features, the analyzer allows one to calculate all the metrics at once.\n\nFirst, instantiate an analyzer object\n\n```python\nfrom pulsar_metrics.analyzers.base import Analyzer\nanalysis = Analyzer(name = \'First Analyzer\', description=\'My first Analyzer\', data = data_new)\n```\n\nThen add the metrics of interest\n\n```python\nanalysis.add_drift_metrics(metrics_list=[\'wasserstein\', \'ttest\', \'ks_2samp\'], features_list=[\'Population\', \'MedInc\']);\nanalysis.add_performance_metrics(metrics_list=[\'accuracy\'], y_name = \'clf_target\');\n```\n\nThen, you can run the analyzer while optionnally specfyings options for each metrics as a dictionnary for the `options` keywords\n\n```python\nanalysis.run(data_ref = data_ref, options = {\'ttest\': {\'alpha\': 0.01, \'equal_var\': False}})\n```\n\nIt then possible to get the results of the analysis as a pandas dataFrame\n\n```python\nanalysis.results_to_pandas()\n```\n\n![image](https://user-images.githubusercontent.com/105239615/206878435-b3bd2b8d-5196-45cd-9eb6-76d70e002c23.png)\n\n#### Creating a custom metric\nThe `@CustomMetric` decorator allows to transform any function to the `AbstractMetrics` class\n\n```python\nfrom pulsar_metrics.metrics.base import CustomMetric\n@CustomMetric\ndef test_custom(a, b, **kwargs):\n    return np.max(a - b)\n```\n\n## About [PulsarML](https://pulsar.ml/)\n\nPulsarML is a project helping with monitoring your models and gain powerful insights into its performance.\n\nWe released two Open Source packages :\n- [pulsar-data-collection](https://github.com/Rocket-Science-Development/pulsar_data_collection) :  lightweight python SDK enabling data collection of features, predictions and metadata from an ML model serving code/micro-service\n- [pulsar-metrics](https://github.com/Rocket-Science-Development/pulsar_metrics) : library for evaluating and monitoring data and concept drift with an extensive set of metrics. It also offers the possibility to use custom metrics defined by the user.\n\nWe also created [pulsar demo](https://github.com/Rocket-Science-Development/pulsar_demo) to display an example use-case showing how to leverage both packages to implement model monitoring and performance management.\n\nWant to interact with the community? join our [slack channel](https://pulsarml.slack.com)\n\nPowered by [Rocket Science Development](https://rocketscience.one/)\n\n## Contributing\n\n1. Fork this repository, develop, and test your changes\n2. open an issue\n3. Submit a pull request with a reference to the issue\n',
    'author': 'Adel Benlagra',
    'author_email': 'abenlagra@rocketscience.one',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Rocket-Science-Development/pulsar_metrics',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)
